#pragma once

#include "internal.h"

#include <stddef.h>

#if defined(WIN32)
#if defined(ACMS_API_EXPORT)
#define ACMS_API __declspec(dllexport)
#else
#define ACMS_API __declspec(dllimport)
#endif
#else
#define ACMS_API
#endif

#if defined(__cplusplus)
extern "C" {
#endif

/* Endpoint_t */
struct Endpoint_t;
ACMS_API const char* Endpoint_get_host(struct Endpoint_t* ep);
ACMS_API void Endpoint_set_host(struct Endpoint_t* ep, const char* host);

ACMS_API const char* Endpoint_get_port(struct Endpoint_t* ep);
ACMS_API void Endpoint_set_port(struct Endpoint_t* ep, const char* port);

/* InitData_t */
struct InitData_t;
ACMS_API struct InitData_t* InitData_new();
ACMS_API void InitData_delete(struct InitData_t* ri);

ACMS_API bool InitData_set_endpoint(struct InitData_t* ri, const char* id, const char* host, const char* port);
ACMS_API bool InitData_set_cert(struct InitData_t* ri,
    const char* cert, size_t cert_len,
    const char* key, size_t key_len,
    const char* ca, size_t ca_len);
ACMS_API bool InitData_set_cert_file(struct InitData_t* ri, const char* cert, const char* key, const char* ca);
ACMS_API bool InitData_set_filter(struct InitData_t* ri, EndpointFilter_t filter_cb, void* cb_ctx);

/* client functions */
ACMS_API int acms_client_init(struct InitData_t* ri,
    bool bProxy,
    const char* local_ip, const char* local_port);
ACMS_API int acms_client_deinit(struct InitData_t* ri);

ACMS_API uint16_t acms_client_localport(struct InitData_t* ri);
ACMS_API int acms_client_devlist_get(struct InitData_t* ri, char** devlist, size_t* size);

/* server functions */
ACMS_API int acms_server_init(struct InitData_t* ri);
ACMS_API int acms_server_deinit(struct InitData_t* ri);

/* logging */
ACMS_API void acms_setlogger(print_logger_t);

#if defined(__cplusplus)
}
#endif
